@@ -11,7 +11,7 @@
 
 #include <compel/plugins/std/asm/syscall-types.h>
 
-#define core_is_compat(core)			false
+#define core_is_compat(core) false
 
 typedef UserArmRegsEntry UserRegsEntry;
 
@@ -21,10 +21,18 @@ typedef UserArmRegsEntry UserRegsEntry;
 
 #define TI_SP(core) ((core)->ti_arm->gpregs->sp)
 
-static inline void *decode_pointer(u64 v) { return (void*)(u32)v; }
-static inline u64 encode_pointer(void *p) { return (u32)p; }
+#define TI_IP(core) ((core)->ti_arm->gpregs->ip)
 
-#define AT_VECTOR_SIZE		40
+static inline void *decode_pointer(u64 v)
+{
+	return (void *)(u32)v;
+}
+static inline u64 encode_pointer(void *p)
+{
+	return (u32)p;
+}
+
+#define AT_VECTOR_SIZE 40
 typedef uint32_t auxv_t;
 typedef uint32_t tls_t;
 
