@@ -1,13 +1,13 @@
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <inttypes.h>
 #include <grp.h>
 #include <string.h>
 #include <errno.h>
 #include <stdbool.h>
 #include <sys/mount.h>
-#include <sys/types.h>
+#include <sys/sysmacros.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/param.h>
@@ -16,15 +16,19 @@
 #include <signal.h>
 #include <sched.h>
 #include <sys/socket.h>
+#include <time.h>
+#include <sys/prctl.h>
 
 #include "zdtmtst.h"
 #include "ns.h"
 
+int criu_status_in = -1, criu_status_in_peer = -1, criu_status_out = -1;
+
 extern int pivot_root(const char *new_root, const char *put_old);
 static int prepare_mntns(void)
 {
 	int dfd, ret;
-	char *root, *criu_path;
+	char *root, *criu_path, *dev_path;
 	char path[PATH_MAX];
 
 	root = getenv("ZDTM_ROOT");
@@ -38,7 +42,7 @@ static int prepare_mntns(void)
 	 * under them. So we need to create another mount for the
 	 * new root.
 	 */
-	if (mount(root, root, NULL, MS_SLAVE , NULL)) {
+	if (mount(root, root, NULL, MS_SLAVE, NULL)) {
 		fprintf(stderr, "Can't bind-mount root: %m\n");
 		return -1;
 	}
@@ -48,11 +52,23 @@ static int prepare_mntns(void)
 		return -1;
 	}
 
+	dev_path = getenv("ZDTM_DEV");
+	if (dev_path) {
+		snprintf(path, sizeof(path), "%s/dev", root);
+		if (mount(dev_path, path, NULL, MS_BIND, NULL)) {
+			pr_perror("Unable to mount %s",  path);
+			return -1;
+		}
+		if (mount(NULL, path, NULL, MS_PRIVATE, NULL)) {
+			pr_perror("Unable to mount %s",  path);
+			return -1;
+		}
+	}
+
 	criu_path = getenv("ZDTM_CRIU");
 	if (criu_path) {
 		snprintf(path, sizeof(path), "%s%s", root, criu_path);
-		if (mount(criu_path, path, NULL, MS_BIND, NULL) ||
-		    mount(NULL, path, NULL, MS_PRIVATE, NULL)) {
+		if (mount(criu_path, path, NULL, MS_BIND, NULL) || mount(NULL, path, NULL, MS_PRIVATE, NULL)) {
 			pr_perror("Unable to mount %s", path);
 			return -1;
 		}
@@ -84,7 +100,7 @@ static int prepare_mntns(void)
 		return -1;
 	}
 
-	if (mount("./old", "./old", NULL, MS_SLAVE | MS_REC , NULL)) {
+	if (mount("./old", "./old", NULL, MS_SLAVE | MS_REC, NULL)) {
 		fprintf(stderr, "Can't bind-mount root: %m\n");
 		return -1;
 	}
@@ -99,6 +115,11 @@ static int prepare_mntns(void)
 		return -1;
 	}
 
+	if (mount("zdtm_run", "/run", "tmpfs", 0, NULL)) {
+		fprintf(stderr, "Unable to mount /run: %m\n");
+		return -1;
+	}
+
 	if (umount2("./old", MNT_DETACH)) {
 		fprintf(stderr, "umount(./old) failed: %m\n");
 		return -1;
@@ -151,7 +172,7 @@ static int prepare_namespaces(void)
 	return 0;
 }
 
-#define NS_STACK_SIZE	4096
+#define NS_STACK_SIZE 4096
 
 /* All arguments should be above stack, because it grows down */
 struct ns_exec_args {
@@ -183,13 +204,11 @@ static void ns_sig_hand(int signo)
 				if (futex_get(&sig_received))
 					return;
 				futex_set_and_wake(&sig_received, signo);
-				len = snprintf(buf, sizeof(buf),
-						"All test processes exited\n");
+				len = snprintf(buf, sizeof(buf), "All test processes exited\n");
 			} else {
-				len = snprintf(buf, sizeof(buf),
-						"wait() failed: %m\n");
+				len = snprintf(buf, sizeof(buf), "wait() failed: %m\n");
 			}
-				goto write_out;
+			goto write_out;
 		}
 		if (status)
 			fprintf(stderr, "%d return %d\n", pid, status);
@@ -201,10 +220,43 @@ write_out:
 	write(STDERR_FILENO, buf, MIN(len, sizeof(buf)));
 }
 
+#ifndef CLONE_NEWTIME
+#define CLONE_NEWTIME 0x00000080 /* New time namespace */
+#endif
+
+static inline int _settime(clockid_t clk_id, time_t offset)
+{
+	int fd, len;
+	char buf[4096];
+
+	if (clk_id == CLOCK_MONOTONIC_COARSE || clk_id == CLOCK_MONOTONIC_RAW)
+		clk_id = CLOCK_MONOTONIC;
+
+	len = snprintf(buf, sizeof(buf), "%d %" PRId64 " 0", clk_id, (int64_t)offset);
+
+	fd = open("/proc/self/timens_offsets", O_WRONLY);
+	if (fd < 0) {
+		fprintf(stderr, "open(/proc/self/timens_offsets): %m");
+		return -1;
+	}
+
+	if (write(fd, buf, len) != len) {
+		fprintf(stderr, "write(/proc/self/timens_offsets): %m");
+		return -1;
+	}
+
+	if (close(fd)) {
+		fprintf(stderr, "close(/proc/self/timens_offsets): %m");
+		return -1;
+	}
+
+	return 0;
+}
+
 #define STATUS_FD 255
 static int ns_exec(void *_arg)
 {
-	struct ns_exec_args *args = (struct ns_exec_args *) _arg;
+	struct ns_exec_args *args = (struct ns_exec_args *)_arg;
 	char buf[4096];
 	int ret;
 
@@ -212,6 +264,7 @@ static int ns_exec(void *_arg)
 
 	setsid();
 
+	prctl(PR_SET_DUMPABLE, 1, 0, 0, 0);
 	ret = dup2(args->status_pipe[1], STATUS_FD);
 	if (ret < 0) {
 		fprintf(stderr, "dup2() failed: %m\n");
@@ -230,24 +283,40 @@ static int ns_exec(void *_arg)
 	return -1;
 }
 
-static void show_ps(void)
+static int create_timens(void)
 {
-	int pid;
+	int fd;
 
-	pid = fork();
-	if (pid == 0) {
-		execl("/bin/ps", "ps", "axf", "-o", "pid,sid,comm", NULL);
-		fprintf(stderr, "Unable to execute ps: %m\n");
+	if (unshare(CLONE_NEWTIME)) {
+		if (errno == EINVAL) {
+			fprintf(stderr, "timens isn't supported\n");
+			return 0;
+		} else {
+			fprintf(stderr, "unshare(CLONE_NEWTIME) failed: %m");
+			exit(1);
+		}
+	}
+
+	if (_settime(CLOCK_MONOTONIC, 10 * 24 * 60 * 60))
 		exit(1);
-	} else if (pid > 0)
-		waitpid(pid, NULL, 0);
+	if (_settime(CLOCK_BOOTTIME, 20 * 24 * 60 * 60))
+		exit(1);
+
+	fd = open("/proc/self/ns/time_for_children", O_RDONLY);
+	if (fd < 0)
+		exit(1);
+	if (setns(fd, 0))
+		exit(1);
+	close(fd);
+
+	return 0;
 }
 
 int ns_init(int argc, char **argv)
 {
 	struct sigaction sa = {
-		.sa_handler	= ns_sig_hand,
-		.sa_flags	= SA_RESTART,
+		.sa_handler = ns_sig_hand,
+		.sa_flags = SA_RESTART,
 	};
 	int ret, fd, status_pipe = STATUS_FD;
 	char buf[128], *x;
@@ -260,6 +329,14 @@ int ns_init(int argc, char **argv)
 		exit(1);
 	}
 
+	if (create_timens())
+		exit(1);
+
+	if (init_notify()) {
+		fprintf(stderr, "Can't init pre-dump notification: %m");
+		exit(1);
+	}
+
 	reap = getenv("ZDTM_NOREAP") == NULL;
 
 	sigemptyset(&sa.sa_mask);
@@ -293,8 +370,6 @@ int ns_init(int argc, char **argv)
 	else if (ret)
 		fprintf(stderr, "The test returned non-zero code %d\n", ret);
 
-	show_ps();
-
 	if (reap && sigaction(SIGCHLD, &sa, NULL)) {
 		fprintf(stderr, "Can't set SIGCHLD handler: %m\n");
 		exit(1);
@@ -308,7 +383,7 @@ int ns_init(int argc, char **argv)
 			break;
 		if (pid < 0) {
 			fprintf(stderr, "waitpid() failed: %m\n");
-			exit (1);
+			exit(1);
 		}
 		if (status)
 			fprintf(stderr, "%d return %d\n", pid, status);
@@ -323,19 +398,17 @@ int ns_init(int argc, char **argv)
 	/* suspend/resume */
 	test_waitsig();
 
-	show_ps();
-
 	fd = open(pidfile, O_RDONLY);
 	if (fd == -1) {
 		fprintf(stderr, "open(%s) failed: %m\n", pidfile);
 		exit(1);
 	}
 	ret = read(fd, buf, sizeof(buf) - 1);
-	buf[ret] = '\0';
 	if (ret == -1) {
 		fprintf(stderr, "read() failed: %m\n");
 		exit(1);
 	}
+	buf[ret] = '\0';
 
 	pid = atoi(buf);
 	fprintf(stderr, "kill(%d, SIGTERM)\n", pid);
@@ -344,17 +417,30 @@ int ns_init(int argc, char **argv)
 
 	ret = 0;
 	if (reap) {
-		while (ret != -1)
-			ret = wait(NULL);
+		while (true) {
+			pid_t child;
+			ret = -1;
+
+			child = waitpid(-1, &ret, 0);
+			if (child < 0) {
+				fprintf(stderr, "Unable to wait a test process: %m");
+				exit(1);
+			}
+			if (child == pid) {
+				fprintf(stderr, "The test returned 0x%x", ret);
+				exit(!(ret == 0));
+			}
+			if (ret)
+				fprintf(stderr, "The %d process exited with 0x%x", child, ret);
+		}
 	} else {
 		waitpid(pid, NULL, 0);
 	}
 
-
 	exit(1);
 }
 
-#define UID_MAP "0 100000 100000\n100000 200000 50000"
+#define UID_MAP "0 20000 20000\n100000 200000 50000"
 #define GID_MAP "0 400000 50000\n50000 500000 100000"
 void ns_create(int argc, char **argv)
 {
@@ -362,6 +448,7 @@ void ns_create(int argc, char **argv)
 	int ret, status;
 	struct ns_exec_args args;
 	int flags;
+	char *pidf;
 
 	args.argc = argc;
 	args.argv = argv;
@@ -372,8 +459,7 @@ void ns_create(int argc, char **argv)
 		exit(1);
 	}
 
-	flags = CLONE_NEWPID | CLONE_NEWNS | CLONE_NEWUTS |
-		CLONE_NEWNET | CLONE_NEWIPC | SIGCHLD;
+	flags = CLONE_NEWPID | CLONE_NEWNS | CLONE_NEWUTS | CLONE_NEWNET | CLONE_NEWIPC | SIGCHLD;
 
 	if (getenv("ZDTM_USERNS"))
 		flags |= CLONE_NEWUSER;
@@ -416,6 +502,14 @@ void ns_create(int argc, char **argv)
 	}
 	shutdown(args.status_pipe[0], SHUT_WR);
 
+	pidf = pidfile;
+	pidfile = malloc(strlen(pidfile) + 13);
+	sprintf(pidfile, "%s%s", pidf, INPROGRESS);
+	if (write_pidfile(pid)) {
+		fprintf(stderr, "Preparations fail\n");
+		exit(1);
+	}
+
 	status = 1;
 	ret = read(args.status_pipe[0], &status, sizeof(status));
 	if (ret != sizeof(status) || status) {
@@ -428,9 +522,11 @@ void ns_create(int argc, char **argv)
 		exit(1);
 	}
 
+	unlink(pidfile);
+	pidfile = pidf;
+
 	if (write_pidfile(pid))
 		exit(1);
 
 	exit(0);
 }
-
